@@ -1,4 +1,4 @@
-/* $Id: appointment.c,v 1.29 2004/08/17 16:10:21 twogood Exp $ */
+/* $Id: appointment.c 2671 2006-11-26 19:19:36Z lilalinux $ */
 #define _GNU_SOURCE 1
 #include "appointment.h"
 #include "appointment_ids.h"
@@ -24,6 +24,7 @@
 
 #define BLOB0067_STR  "BLOB0067="
 
+#if 0
 static const uint8_t BLOB0067_HEADER[20] = { 
   0x04, 0x00, 0x00, 0x00, 
   0x82, 0x00, 0xe0, 0x00,
@@ -31,6 +32,7 @@ static const uint8_t BLOB0067_HEADER[20] = {
   0x1a, 0x82, 0xe0, 0x08,
   0x00, 0x00, 0x00, 0x00 
 };
+#endif
 
 typedef struct _EventGeneratorData
 {
@@ -41,6 +43,7 @@ typedef struct _EventGeneratorData
   CEPROPVAL* reminder_enabled;
 #if ENABLE_RECURRENCE
   CEPROPVAL* recurrence_pattern;
+  CEPROPVAL* recurrence_timezone;
   CEPROPVAL* unique;
 #endif
 } EventGeneratorData;
@@ -115,6 +118,17 @@ static bool on_propval_recurrence_pattern(Generator* g, CEPROPVAL* propval, void
   return true;
 }
 
+static bool on_propval_recurrence_timezone(Generator* g, CEPROPVAL* propval, void* cookie)
+{
+#if ENABLE_RECURRENCE
+  EventGeneratorData* data = (EventGeneratorData*)cookie;
+  data->recurrence_timezone = propval;
+#else
+  synce_warning("Recurrence support not enabled");
+#endif
+  return true;
+}
+
 static bool on_propval_start(Generator* g, CEPROPVAL* propval, void* cookie)
 {
   EventGeneratorData* data = (EventGeneratorData*)cookie;
@@ -172,6 +186,7 @@ bool rra_appointment_to_vevent(/*{{{*/
   generator_add_property(generator, ID_SENSITIVITY, on_propval_sensitivity);
   generator_add_property(generator, ID_APPOINTMENT_START,       on_propval_start);
   generator_add_property(generator, ID_RECURRENCE_PATTERN, on_propval_recurrence_pattern);
+  generator_add_property(generator, ID_RECURRENCE_TIMEZONE, on_propval_recurrence_timezone);
   generator_add_property(generator, ID_SUBJECT,     on_propval_subject);
 #if ENABLE_RECURRENCE
   generator_add_property(generator, ID_UNIQUE,      on_propval_unique);
@@ -210,15 +225,19 @@ bool rra_appointment_to_vevent(/*{{{*/
       event_generator_data.type)
   {
     char buffer[32];
-    time_t start_time = 
-      filetime_to_unix_time(&event_generator_data.start->val.filetime);
+    time_t start_time = 0;
     time_t end_time = 0;
     const char* type = NULL;
     const char* format = NULL;
-    
+    struct tm* (*xtime)(const time_t *timep) = NULL;
+
+    if (!parser_filetime_to_unix_time(&event_generator_data.start->val.filetime, &start_time))
+      goto exit;
+
     switch (event_generator_data.type->val.lVal)
     {
       case APPOINTMENT_TYPE_ALL_DAY:
+        xtime  = localtime;
         type   = "DATE";
         format = "%Y%m%d";
 
@@ -230,23 +249,22 @@ bool rra_appointment_to_vevent(/*{{{*/
 
 
       case APPOINTMENT_TYPE_NORMAL:
+        xtime  = gmtime;
         type   = "DATE-TIME";
-        /*if (tzi)*/
+        if (!tzi)
           format = "%Y%m%dT%H%M%SZ";
-        /*else
-          format = "%Y%m%dT%H%M%S";*/
+        else
+          format = "%Y%m%dT%H%M%S";
 
         /* minutes to seconds */
         end_time = start_time + 
           event_generator_data.duration->val.lVal * 60;
 
-#if 0
         if (tzi)
         {
-          start_time = rra_timezone_convert_to_utc(tzi, start_time);
-          end_time   = rra_timezone_convert_to_utc(tzi, end_time);
+          start_time = rra_timezone_convert_from_utc(tzi, start_time);
+          end_time   = rra_timezone_convert_from_utc(tzi, end_time);
         }
-#endif
       
         break;
 
@@ -258,14 +276,12 @@ bool rra_appointment_to_vevent(/*{{{*/
 
     if (type && format)
     {
-      /* using localtime here to get correct output from strftime */
-      strftime(buffer, sizeof(buffer), format, localtime(&start_time));
+      strftime(buffer, sizeof(buffer), format, xtime(&start_time));
       generator_add_with_type(generator, "DTSTART", type, buffer);
       
       if (end_time)
       {
-        /* using localtime here to get correct output from strftime */
-        strftime(buffer, sizeof(buffer), format, localtime(&end_time));
+        strftime(buffer, sizeof(buffer), format, xtime(&end_time));
         generator_add_with_type(generator, "DTEND",   type, buffer);
       }
     }
@@ -273,37 +289,10 @@ bool rra_appointment_to_vevent(/*{{{*/
   else
     synce_warning("Missing start, duration or duration unit");
 
-
-  if (event_generator_data.reminder_enabled &&
-      event_generator_data.reminder_minutes &&
-      event_generator_data.reminder_enabled->val.iVal)
-  {
-    char buffer[32];
-
-    generator_add_simple(generator, "BEGIN", "VALARM");
-
-    /* XXX: maybe this should correspond to ID_REMINDER_OPTIONS? */
-    generator_add_simple(generator, "ACTION", "DISPLAY");
-
-    /* XXX: what if minutes > 59 */
-    snprintf(buffer, sizeof(buffer), "-PT%liM", 
-        event_generator_data.reminder_minutes->val.lVal);
-
-    generator_begin_line         (generator, "TRIGGER");
-    
-    generator_begin_parameter    (generator, "VALUE");
-    generator_add_parameter_value(generator, "DURATION");
-    generator_end_parameter      (generator);
-    
-    generator_begin_parameter    (generator, "RELATED");
-    generator_add_parameter_value(generator, "START");
-    generator_end_parameter      (generator);
-
-    generator_add_value          (generator, buffer);
-    generator_end_line           (generator);
-    
-    generator_add_simple(generator, "END", "VALARM");
-  }
+  to_icalendar_trigger(generator,
+                       event_generator_data.reminder_enabled,
+                       event_generator_data.reminder_minutes,
+                       REMINDER_RELATED_START);
 
 #if ENABLE_RECURRENCE
   if (event_generator_data.recurrence_pattern)
@@ -311,7 +300,7 @@ bool rra_appointment_to_vevent(/*{{{*/
     if (!recurrence_generate_rrule(generator, event_generator_data.recurrence_pattern))
       synce_warning("Failed to generate RRULE from recurrence pattern.");
 
-    if (event_generator_data.unique)
+    if (event_generator_data.unique && id == RRA_APPOINTMENT_ID_UNKNOWN)
     {
       char* buffer = NULL;
       unsigned i;
@@ -385,9 +374,9 @@ exit:
 
 typedef struct _EventParserData
 {
-  bool has_alarm;
   mdir_line* dtstart;
   mdir_line* dtend;
+  mdir_line* trigger;
 #if ENABLE_RECURRENCE
   RRA_MdirLineVector* exdates;
   mdir_line* rrule;
@@ -397,65 +386,15 @@ typedef struct _EventParserData
 
 static bool on_timezone_tzid(Parser* p, mdir_line* line, void* cookie)
 {
-  synce_trace("TZID = '%s'", line->values[0]);
+  if (line)
+    synce_trace("TZID = '%s'", line->values[0]);
   return true;
 }
 
 static bool on_alarm_trigger(Parser* p, mdir_line* line, void* cookie)/*{{{*/
 {
   EventParserData* event_parser_data = (EventParserData*)cookie;
-
-  char** data_type = mdir_get_param_values(line, "VALUE");
-  char** related   = mdir_get_param_values(line, "RELATED");
-  int duration = 0;
-
-  if (event_parser_data->has_alarm)
-    goto exit;
-
-  /* data type must be DURATION */
-  if (data_type && data_type[0])
-  {
-    if (STR_EQUAL(data_type[0], "DATE-TIME"))
-    {
-      synce_warning("Absolute date/time for alarm is not supported");
-      goto exit;
-    }
-    if (!STR_EQUAL(data_type[0], "DURATION"))
-    {
-      synce_warning("Unknown TRIGGER data type: '%s'", data_type[0]);
-      goto exit;
-    }
-  }
-
-  /* related must be START */
-  if (related && related[0])
-  {
-    if (STR_EQUAL(related[0], "END"))
-    {
-      synce_warning("Alarms related to event end are not supported");
-      goto exit;
-    }
-    if (!STR_EQUAL(related[0], "START"))
-    {
-      synce_warning("Unknown TRIGGER data type: '%s'", related[0]);
-      goto exit;
-    }
-  }
-
-  if (parser_duration_to_seconds(line->values[0], &duration) && duration <= 0)
-  {
-    /* XXX: use ACTION for this instead of defaults? */
-    parser_add_int32(p, ID_REMINDER_OPTIONS, REMINDER_LED|REMINDER_DIALOG|REMINDER_SOUND);
-
-    /* set alarm */
-    parser_add_int32(p, ID_REMINDER_MINUTES_BEFORE_START, -duration / 60);
-    parser_add_int16(p, ID_REMINDER_ENABLED, 1);
-    parser_add_string(p, ID_REMINDER_SOUND_FILE, "Alarm1.wav");
-
-    event_parser_data->has_alarm = true;
-  }
-
-exit:
+  event_parser_data->trigger = line;
   return true;
 }/*}}}*/
 
@@ -477,7 +416,8 @@ static bool on_mdir_line_dtstart(Parser* p, mdir_line* line, void* cookie)/*{{{*
 static bool on_mdir_line_exdate(Parser* p, mdir_line* line, void* cookie)/*{{{*/
 {
   EventParserData* event_parser_data = (EventParserData*)cookie;
-  rra_mdir_line_vector_add(event_parser_data->exdates, line);
+  if (line)
+    rra_mdir_line_vector_add(event_parser_data->exdates, line);
   return true;
 }/*}}}*/
 #endif
@@ -495,7 +435,7 @@ static bool on_mdir_line_rrule(Parser* p, mdir_line* line, void* cookie)/*{{{*/
 
 static bool on_mdir_line_transp(Parser* p, mdir_line* line, void* cookie)/*{{{*/
 {
-  if (STR_EQUAL(line->values[0], "OPAQUE"))
+  if (!line || STR_EQUAL(line->values[0], "OPAQUE"))
     parser_add_int16(p, ID_BUSY_STATUS, BUSY_STATUS_BUSY);
   else if (STR_EQUAL(line->values[0], "TRANSPARENT"))
     parser_add_int16(p, ID_BUSY_STATUS, BUSY_STATUS_FREE);
@@ -615,6 +555,8 @@ bool rra_appointment_from_vevent(/*{{{*/
     goto exit;
   }
 
+  parser_call_unused_properties(parser);
+
   if (event_parser_data.dtstart)
   {
     if (!parser_add_time_from_line(parser, ID_APPOINTMENT_START, event_parser_data.dtstart))
@@ -666,6 +608,7 @@ bool rra_appointment_from_vevent(/*{{{*/
         synce_warning("Failed to parse recurrence rule");
 
       if (event_parser_data.uid)
+      {
         if (0 == strncmp(event_parser_data.uid->values[0], BLOB0067_STR, strlen(BLOB0067_STR)))
         {
           /* A binary UID from SynCE */
@@ -694,6 +637,7 @@ bool rra_appointment_from_vevent(/*{{{*/
               (uint8_t*)event_parser_data.uid->values[0], 
               strlen(event_parser_data.uid->values[0]));
         }
+      }
     }
     else
 #endif
@@ -705,15 +649,8 @@ bool rra_appointment_from_vevent(/*{{{*/
     goto exit;
   }
 
-  if (!event_parser_data.has_alarm)
-  {
-    /* default stuff */
-    parser_add_int16 (parser, ID_REMINDER_ENABLED, 0);
-    parser_add_int32 (parser, ID_REMINDER_MINUTES_BEFORE_START, 15);
-    parser_add_int32 (parser, ID_REMINDER_OPTIONS, REMINDER_LED|REMINDER_DIALOG|REMINDER_SOUND);
-    parser_add_string(parser, ID_REMINDER_SOUND_FILE, "Alarm1.wav");
-  }
-  
+  to_propval_trigger(parser, event_parser_data.trigger, REMINDER_RELATED_START);
+
   /* The calendar application on my HP 620LX just hangs without this! */
   parser_add_int32(parser, ID_UNKNOWN_0002, 0);
 
